@@ -1,3 +1,4 @@
+import platform
 import os
 import time
 
@@ -31,7 +32,7 @@ def plugin(kernel, lifecycle=None):
 
 class Driver:
     """
-    An Driver takes spoolable commands and turns those commands into states and code in a language
+    A driver takes spoolable commands and turns those commands into states and code in a language
     agnostic fashion. This is intended to be overridden by a subclass or class with the required methods.
 
     These drive hardware specific backend information from the reusable spoolers and server objects that may also be
@@ -64,7 +65,6 @@ class Driver:
         self.plot = None
 
         self.state = DRIVER_STATE_RAPID
-        self.properties = 0
         self.is_relative = False
         self.laser = False
         self.root_context.setting(bool, "opt_rapid_between", True)
@@ -72,8 +72,6 @@ class Driver:
         self.root_context.setting(int, "opt_jog_minimum", 127)
         context._quit = False
 
-        self.rapid = self.root_context.opt_rapid_between
-        self.jog = self.root_context.opt_jog_mode
         self.rapid_override = False
         self.rapid_override_speed_x = 50.0
         self.rapid_override_speed_y = 50.0
@@ -82,6 +80,8 @@ class Driver:
         self.context.kernel.listen("lifecycle;ready", "", self.start_driver)
         self.context.kernel.listen("lifecycle;shutdown", "", self.shutdown)
 
+        self.last_fetch = None
+
     def shutdown(self, *args, **kwargs):
         self.context.kernel.unlisten("lifecycle;ready", "", self.start_driver)
         self.context.kernel.unlisten("lifecycle;shutdown", "", self.shutdown)
@@ -138,6 +138,7 @@ class Driver:
         # We have a spooled item to process.
         if self.command(self.spooled_item):
             self.spooled_item = None
+            self.spooler.pop()
             return
 
         # We are dealing with an iterator/generator
@@ -148,6 +149,7 @@ class Driver:
         except StopIteration:
             # The spooled item is finished.
             self.spooled_item = None
+            self.spooler.pop()
 
     def plotplanner_process(self):
         """
@@ -167,17 +169,24 @@ class Driver:
         if self.spooler is None:
             return  # Spooler does not exist.
         element = self.spooler.peek()
+
+        if self.last_fetch is not None:
+            self.context.channel("spooler")(
+                "Time between fetches: %f" % (time.time() - self.last_fetch)
+            )
+            self.last_fetch = None
+
         if element is None:
             return  # Spooler is empty.
 
-        self.spooler.pop()
+        self.last_fetch = time.time()
+
+        # self.spooler.pop()
         if isinstance(element, int):
             self.spooled_item = (element,)
         elif isinstance(element, tuple):
             self.spooled_item = element
         else:
-            self.rapid = self.root_context.opt_rapid_between
-            self.jog = self.root_context.opt_jog_mode
             try:
                 self.spooled_item = element.generate()
             except AttributeError:
@@ -227,6 +236,8 @@ class Driver:
                 self.unlock_rail()
             elif command == COMMAND_PLOT:
                 self.plot_plot(values[0])
+            elif command == COMMAND_BLOB:
+                self.send_blob(values[0], values[1])
             elif command == COMMAND_PLOT_START:
                 self.plot_start()
             elif command == COMMAND_SET_SPEED:
@@ -254,19 +265,20 @@ class Driver:
             elif command == COMMAND_SET_POSITION:
                 self.set_position(values[0], values[1])
             elif command == COMMAND_MODE_RAPID:
-                self.ensure_rapid_mode()
+                self.ensure_rapid_mode(*values)
             elif command == COMMAND_MODE_PROGRAM:
-                self.ensure_program_mode()
+                self.ensure_program_mode(*values)
             elif command == COMMAND_MODE_RASTER:
-                self.ensure_raster_mode()
+                self.ensure_raster_mode(*values)
             elif command == COMMAND_MODE_FINISHED:
-                self.ensure_finished_mode()
+                self.ensure_finished_mode(*values)
             elif command == COMMAND_WAIT:
                 self.wait(values[0])
             elif command == COMMAND_WAIT_FINISH:
                 self.wait_finish()
             elif command == COMMAND_BEEP:
-                if os.name == "nt":
+                OS_NAME = platform.system()
+                if OS_NAME == "Windows":
                     try:
                         import winsound
 
@@ -274,17 +286,19 @@ class Driver:
                             winsound.Beep(2000, 100)
                     except Exception:
                         pass
-                if os.name == "posix":
-                    # Mac or linux.
-                    print("\a")  # Beep.
-                    os.system('say "Ding"')
-                else:
+                elif OS_NAME == "Darwin": # Mac
+                    os.system('afplay /System/Library/Sounds/Ping.aiff')
+                else: # Assuming other linux like system
                     print("\a")  # Beep.
             elif command == COMMAND_FUNCTION:
                 if len(values) >= 1:
                     t = values[0]
                     if callable(t):
                         t()
+            elif command == COMMAND_CONSOLE:
+                if len(values) == 1:
+                    fn = self.context.console_function(values[0])
+                    fn()
             elif command == COMMAND_SIGNAL:
                 if isinstance(values, str):
                     self.context.signal(values, None)
@@ -497,21 +511,6 @@ class Driver:
         status = ";".join(parts)
         self.context.signal("driver;status", status)
 
-    def set_prop(self, mask):
-        self.properties |= mask
-
-    def unset_prop(self, mask):
-        self.properties &= ~mask
-
-    def is_prop(self, mask):
-        return bool(self.properties & mask)
-
-    def toggle_prop(self, mask):
-        if self.is_prop(mask):
-            self.unset_prop(mask)
-        else:
-            self.set_prop(mask)
-
 
 class Drivers(Modifier):
     def __init__(self, context, name=None, channel=None, *args, **kwargs):
@@ -531,6 +530,7 @@ class Drivers(Modifier):
         except KeyError:
             device = [None, None, None]
             self.context.registered[dev] = device
+            self.context.signal("legacy_spooler_label", device_name)
         if device[1] is not None and driver_type is None:
             return device[1]
         try:
@@ -538,6 +538,7 @@ class Drivers(Modifier):
                 driver_class = self.context.registered[itype]
                 driver = driver_class(self.context, device_name, **kwargs)
                 device[1] = driver
+                self.context.signal("legacy_spooler_label", device_name)
                 return driver
         except (KeyError, IndexError):
             return None
