@@ -5,8 +5,11 @@ from ..device.basedevice import (
     PLOT_DIRECTION,
     PLOT_FINISH,
     PLOT_JOG,
+    PLOT_LEFT_UPPER,
     PLOT_RAPID,
+    PLOT_RIGHT_LOWER,
     PLOT_SETTING,
+    PLOT_START,
 )
 
 """
@@ -65,8 +68,31 @@ class PlotPlanner:
         """
         Main method of generating the plot stream.
 
+        The plotplanner emits a series of 3 item values. These are x, y and on. For most plotting
+        the values for on is 0 or 1 for off and on respectively. Values for on greater than 0 give
+        additional information. This includes an initial jog. Settings change. Major axis. Plot direction.
+        left_upper position and right_lower position.
+
+        * If new position is not coincident with previous position
+             * If new position is close
+                  * Plot line if close within current settings
+        * Flush steps from planner if going to new location or using new settings.
+             * If new position is far
+                  * Send jog if too far away.
+        * Send PLOT_SETTING: None None - if settings have changed.
+        * Send PLOT_AXIS: major_axis, None - Major axis is horizontal vs. vertical raster
+        * Send PLOT_DIRECTION: x_dir(), y_dir() - Direction X, Direction Y for initial cut.
+        * Send PLOT_LEFT_UPPER: left point, upper point - Point at upper left
+        * Send PLOT_RIGHT_LOWER: right point, left point - Point at lower right
+        * Send all X, Y points for cut.
+
+        If the next position is too far away and jogging is allowed we jog to the position. This
+        jog is sent after the previous data is planner is
+
         :return:
         """
+        self.pos_x = None
+        self.pos_y = None
         while len(self.queue):
             cut = self.queue.pop(0)
             cut_set = cut.settings
@@ -92,10 +118,13 @@ class PlotPlanner:
                         and abs(self.pos_y - new_start_y) < distance
                     ) or not cur_set.jog_enable:
                         # Jog distance smaller than threshold. Or jog isn't allowed
-                        yield from self.step_move(
-                            self.pos_x, self.pos_y, new_start_x, new_start_y
-                        )
-                        self.warp(new_start_x, new_start_y)
+                        def walk():
+                            for event in ZinglPlotter.plot_line(
+                                self.pos_x, self.pos_y, new_start_x, new_start_y
+                            ):
+                                yield event[0], event[1], 0
+
+                        yield from self.process_plots(walk())
                     else:
                         # Request standard jog new location required.
                         flush = True
@@ -120,13 +149,16 @@ class PlotPlanner:
             if cut_set is not cur_set:
                 yield None, None, PLOT_SETTING
 
-            if jog:
+            if jog or cut_set.raster_step != 0:
                 # set the directions. Post Jog, Post Settings.
                 yield cut.major_axis(), None, PLOT_AXIS
                 yield cut.x_dir(), cut.y_dir(), PLOT_DIRECTION
+                yield cut.left(), cut.upper(), PLOT_LEFT_UPPER
+                yield cut.right(), cut.lower(), PLOT_RIGHT_LOWER
 
             # Plot the current.
             # Current is executed in cut settings.
+            yield None, None, PLOT_START
             yield from self.process_plots(cut.generator())
             self.pos_x = self.single.single_x
             self.pos_y = self.single.single_y
@@ -155,6 +187,7 @@ class PlotPlanner:
         """
         # Applies single, ppi, shift, then group.
         if self.debug:
+
             def debug(plot, manipulator):
                 for q in plot:
                     print("Manipulator: %s, %s" % (str(q), str(manipulator)))
@@ -450,7 +483,11 @@ class Group(PlotManipulation):
         )
 
     def flushed(self):
-        return self.last_x == self.group_x and self.last_y == self.group_y and self.last_on == self.group_on
+        return (
+            self.last_x == self.group_x
+            and self.last_y == self.group_y
+            and self.last_on == self.group_on
+        )
 
     def process(self, plot):
         """
@@ -541,3 +578,41 @@ class Group(PlotManipulation):
         self.group_y = None
         self.group_dx = 0
         self.group_dy = 0
+
+
+def grouped(plot):
+    """
+    Converts a generated series of single stepped plots into grouped orthogonal/diagonal plots.
+
+    :param plot: single stepped plots to be grouped into orth/diag sequences.
+    :return:
+    """
+    group_x = None
+    group_y = None
+    group_dx = 0
+    group_dy = 0
+
+    for x, y in plot:
+        if group_x is None:
+            group_x = x
+        if group_y is None:
+            group_y = y
+        if x == group_x + group_dx and y == group_y + group_dy:
+            # This is an orthogonal/diagonal step along the same path.
+            group_x = x
+            group_y = y
+            continue
+        # This is non orth-diag point. Must drop a point.
+        yield group_x, group_y
+        # If we do not have a defined direction, set our current direction.
+        group_dx = x - group_x
+        group_dy = y - group_y
+        if abs(group_dx) > 1 or abs(group_dy) > 1:
+            # The last step was not valid.
+            raise ValueError(
+                "dx(%d) or dy(%d) exceeds 1" % (group_dx, group_dy)
+            )
+        group_x = x
+        group_y = y
+    # There are no more plots.
+    yield group_x, group_y
