@@ -4,6 +4,7 @@ from meerk40t.svgelements import Length
 
 from ..device.lasercommandconstants import *
 from ..kernel import Modifier
+from .elements import MILS_IN_MM
 
 
 def plugin(kernel, lifecycle=None):
@@ -42,6 +43,9 @@ class Spooler:
         self._queue = None
         self.next = None
 
+    def __len__(self):
+        return len(self._queue)
+
     def as_device(self):
         links = []
         obj = self
@@ -64,6 +68,7 @@ class Spooler:
 
     def pop(self):
         if len(self._queue) == 0:
+            self.context.signal("spooler;queue", len(self._queue), self.name)
             return None
         self.queue_lock.acquire(True)
         queue_head = self._queue[0]
@@ -120,9 +125,12 @@ class Spooler:
         self.queue_lock.release()
         self.context.signal("spooler;queue", len(self._queue))
 
-    def remove(self, element):
+    def remove(self, element, index=None):
         self.queue_lock.acquire(True)
-        self._queue.remove(element)
+        if index is None:
+            self._queue.remove(element)
+        else:
+            del self._queue[index]
         self.queue_lock.release()
         self.context.signal("spooler;queue", len(self._queue))
 
@@ -138,6 +146,7 @@ class Spoolers(Modifier):
         except KeyError:
             device = [None, None, None]
             self.context.registered[dev] = device
+            self.context.signal("legacy_spooler_label", device_name)
         if device[0] is None:
             device[0] = Spooler(self.context, device_name)
         return device[0]
@@ -149,22 +158,27 @@ class Spoolers(Modifier):
         context = self.context
         context.spoolers = self
         bed_dim = context.root
+        self.context.root.setting(str, "active", "0")
 
         kernel = self.context._kernel
         _ = kernel.translation
 
         @context.console_option(
-            "register", "r", type=bool, action="store_true", help="Register this device"
+            "register",
+            "r",
+            type=bool,
+            action="store_true",
+            help=_("Register this device"),
         )
         @context.console_command(
             "spool",
-            help="spool<?> <command>",
+            help=_("spool<?> <command>"),
             regex=True,
             input_type=(None, "plan", "device"),
             output_type="spooler",
         )
         def spool(
-            command, channel, _, data=None, register=False, remainder=None, **kwargs
+            command, channel, _, data=None, register=False, remainder=None, **kwgs
         ):
             root = self.context.root
             if len(command) > 5:
@@ -189,52 +203,51 @@ class Spoolers(Modifier):
             spooler = self.get_or_make_spooler(device_name)
             if data is not None:
                 # If plan data is in data, then we copy that and move on to next step.
-                plan, original, commands, plan_name = data
-                spooler.jobs(plan)
+                spooler.jobs(data.plan)
                 channel(_("Spooled Plan."))
-                self.context.signal("plan", plan_name, 6)
+                self.context.signal("plan", data.name, 6)
 
             if remainder is None:
                 channel(_("----------"))
                 channel(_("Spoolers:"))
-                for i, d_name in enumerate(self.context.match("device", True)):
-                    channel("%d: %s" % (i, d_name))
+                for d, d_name in enumerate(self.context.match("device", True)):
+                    channel("%d: %s" % (d, d_name))
                 channel(_("----------"))
                 channel(_("Spooler %s:" % device_name))
-                for i, op_name in enumerate(spooler.queue):
-                    channel("%d: %s" % (i, op_name))
+                for s, op_name in enumerate(spooler.queue):
+                    channel("%d: %s" % (s, op_name))
                 channel(_("----------"))
 
             return "spooler", (spooler, device_name)
 
         @context.console_command(
             "list",
-            help="spool<?> list",
+            help=_("spool<?> list"),
             input_type="spooler",
             output_type="spooler",
         )
-        def spooler_list(command, channel, _, data_type=None, data=None, **kwargs):
+        def spooler_list(command, channel, _, data_type=None, data=None, **kwgs):
             spooler, device_name = data
             channel(_("----------"))
             channel(_("Spoolers:"))
-            for i, d_name in enumerate(self.context.match("device", True)):
-                channel("%d: %s" % (i, d_name))
+            for d, d_name in enumerate(self.context.match("device", True)):
+                channel("%d: %s" % (d, d_name))
             channel(_("----------"))
             channel(_("Spooler %s:" % device_name))
-            for i, op_name in enumerate(spooler.queue):
-                channel("%d: %s" % (i, op_name))
+            for s, op_name in enumerate(spooler.queue):
+                channel("%d: %s" % (s, op_name))
             channel(_("----------"))
             return data_type, data
 
-        @context.console_argument("op", type=str, help="unlock, origin, home, etc")
+        @context.console_argument("op", type=str, help=_("unlock, origin, home, etc"))
         @context.console_command(
             "send",
-            help="send a plan-command to the spooler",
+            help=_("send a plan-command to the spooler"),
             input_type="spooler",
             output_type="spooler",
         )
         def spooler_send(
-            command, channel, _, data_type=None, op=None, data=None, **kwargs
+            command, channel, _, data_type=None, op=None, data=None, **kwgs
         ):
             spooler, device_name = data
             if op is None:
@@ -251,21 +264,21 @@ class Spoolers(Modifier):
 
         @context.console_command(
             "clear",
-            help="spooler<?> clear",
+            help=_("spooler<?> clear"),
             input_type="spooler",
             output_type="spooler",
         )
-        def spooler_clear(command, channel, _, data_type=None, data=None, **kwargs):
+        def spooler_clear(command, channel, _, data_type=None, data=None, **kwgs):
             spooler, device_name = data
             spooler.clear_queue()
             return data_type, data
 
         def execute_absolute_position(position_x, position_y):
             x_pos = Length(position_x).value(
-                ppi=1000.0, relative_length=bed_dim.bed_width * 39.3701
+                ppi=1000.0, relative_length=bed_dim.bed_width * MILS_IN_MM
             )
             y_pos = Length(position_y).value(
-                ppi=1000.0, relative_length=bed_dim.bed_height * 39.3701
+                ppi=1000.0, relative_length=bed_dim.bed_height * MILS_IN_MM
             )
 
             def move():
@@ -277,10 +290,10 @@ class Spoolers(Modifier):
 
         def execute_relative_position(position_x, position_y):
             x_pos = Length(position_x).value(
-                ppi=1000.0, relative_length=bed_dim.bed_width * 39.3701
+                ppi=1000.0, relative_length=bed_dim.bed_width * MILS_IN_MM
             )
             y_pos = Length(position_y).value(
-                ppi=1000.0, relative_length=bed_dim.bed_height * 39.3701
+                ppi=1000.0, relative_length=bed_dim.bed_height * MILS_IN_MM
             )
 
             def move():
@@ -296,9 +309,9 @@ class Spoolers(Modifier):
             hidden=True,
             input_type=("spooler", None),
             output_type="spooler",
-            help="turn laser on in place",
+            help=_("turn laser on in place"),
         )
-        def plus_laser(data, **kwargs):
+        def plus_laser(data, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
@@ -310,9 +323,9 @@ class Spoolers(Modifier):
             hidden=True,
             input_type=("spooler", None),
             output_type="spooler",
-            help="turn laser off in place",
+            help=_("turn laser off in place"),
         )
-        def minus_laser(data, **kwargs):
+        def minus_laser(data, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
@@ -320,22 +333,22 @@ class Spoolers(Modifier):
             return "spooler", data
 
         @context.console_argument(
-            "amount", type=Length, help="amount to move in the set direction."
+            "amount", type=Length, help=_("amount to move in the set direction.")
         )
         @context.console_command(
             ("left", "right", "up", "down"),
             input_type=("spooler", None),
             output_type="spooler",
-            help="cmd <amount>",
+            help=_("cmd <amount>"),
         )
-        def direction(command, channel, _, data=None, amount=None, **kwargs):
+        def direction(command, channel, _, data=None, amount=None, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
             if amount is None:
                 amount = Length("1mm")
-            max_bed_height = bed_dim.bed_height * 39.3701
-            max_bed_width = bed_dim.bed_width * 39.3701
+            max_bed_height = bed_dim.bed_height * MILS_IN_MM
+            max_bed_width = bed_dim.bed_width * MILS_IN_MM
             if not hasattr(spooler, "_dx"):
                 spooler._dx = 0
             if not hasattr(spooler, "_dy"):
@@ -351,14 +364,15 @@ class Spoolers(Modifier):
             context(".timer 1 0 spool%s jog\n" % device_name)
             return "spooler", data
 
+        @context.console_option("force", "f", type=bool, action="store_true")
         @context.console_command(
             "jog",
             hidden=True,
             input_type="spooler",
             output_type="spooler",
-            help="executes outstanding jog buffer",
+            help=_("executes outstanding jog buffer"),
         )
-        def jog(command, channel, _, data, **kwargs):
+        def jog(command, channel, _, data, force=False, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
@@ -369,65 +383,76 @@ class Spoolers(Modifier):
                 return
             if idx == 0 and idy == 0:
                 return
-            if spooler.job_if_idle(execute_relative_position(idx, idy)):
-                channel(_("Position moved: %d %d") % (idx, idy))
-                spooler._dx -= idx
-                spooler._dy -= idy
+            if force:
+                spooler.job(execute_relative_position(idx, idy))
             else:
-                channel(_("Busy Error"))
+                if spooler.job_if_idle(execute_relative_position(idx, idy)):
+                    channel(_("Position moved: %d %d") % (idx, idy))
+                    spooler._dx -= idx
+                    spooler._dy -= idy
+                else:
+                    channel(_("Busy Error"))
             return "spooler", data
 
-        @context.console_argument("x", type=Length, help="change in x")
-        @context.console_argument("y", type=Length, help="change in y")
+        @context.console_option("force", "f", type=bool, action="store_true")
+        @context.console_argument("x", type=Length, help=_("change in x"))
+        @context.console_argument("y", type=Length, help=_("change in y"))
         @context.console_command(
             ("move", "move_absolute"),
             input_type=("spooler", None),
             output_type="spooler",
-            help="move <x> <y>: move to position.",
+            help=_("move <x> <y>: move to position."),
         )
-        def move(channel, _, x, y, data=None, **kwargs):
+        def move(channel, _, x, y, data=None, force=False, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
             if y is None:
                 raise SyntaxError
-            if not spooler.job_if_idle(execute_absolute_position(x, y)):
-                channel(_("Busy Error"))
+            if force:
+                spooler.job(execute_absolute_position(x, y))
+            else:
+                if not spooler.job_if_idle(execute_absolute_position(x, y)):
+                    channel(_("Busy Error"))
             return "spooler", data
 
-        @context.console_argument("dx", type=Length, help="change in x")
-        @context.console_argument("dy", type=Length, help="change in y")
+        @context.console_option("force", "f", type=bool, action="store_true")
+        @context.console_argument("dx", type=Length, help=_("change in x"))
+        @context.console_argument("dy", type=Length, help=_("change in y"))
         @context.console_command(
             "move_relative",
             input_type=("spooler", None),
             output_type="spooler",
-            help="move_relative <dx> <dy>",
+            help=_("move_relative <dx> <dy>"),
         )
-        def move_relative(channel, _, dx, dy, data=None, **kwargs):
+        def move_relative(channel, _, dx, dy, data=None, force=False, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
             if dy is None:
                 raise SyntaxError
-            if not spooler.job_if_idle(execute_relative_position(dx, dy)):
-                channel(_("Busy Error"))
+            if force:
+                spooler.job(execute_relative_position(dx, dy))
+            else:
+                if not spooler.job_if_idle(execute_relative_position(dx, dy)):
+                    channel(_("Busy Error"))
             return "spooler", data
 
-        @context.console_argument("x", type=Length, help="x offset")
-        @context.console_argument("y", type=Length, help="y offset")
+        @context.console_argument("x", type=Length, help=_("x offset"))
+        @context.console_argument("y", type=Length, help=_("y offset"))
         @context.console_command(
             "home",
             input_type=("spooler", None),
             output_type="spooler",
-            help="home the laser",
+            help=_("home the laser"),
         )
-        def home(x=None, y=None, data=None, **kwargs):
+        def home(x=None, y=None, data=None, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
             if x is not None and y is not None:
-                x = x.value(ppi=1000.0, relative_length=bed_dim.bed_width * 39.3701)
-                y = y.value(ppi=1000.0, relative_length=bed_dim.bed_height * 39.3701)
+                x = x.value(ppi=1000.0, relative_length=bed_dim.bed_width * MILS_IN_MM)
+                y = y.value(ppi=1000.0, relative_length=bed_dim.bed_height * MILS_IN_MM)
                 spooler.job(COMMAND_HOME, int(x), int(y))
                 return "spooler", data
             spooler.job(COMMAND_HOME)
@@ -437,9 +462,9 @@ class Spoolers(Modifier):
             "unlock",
             input_type=("spooler", None),
             output_type="spooler",
-            help="unlock the rail",
+            help=_("unlock the rail"),
         )
-        def unlock(data=None, **kwargs):
+        def unlock(data=None, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
@@ -450,9 +475,9 @@ class Spoolers(Modifier):
             "lock",
             input_type=("spooler", None),
             output_type="spooler",
-            help="lock the rail",
+            help=_("lock the rail"),
         )
-        def lock(data, **kwargs):
+        def lock(data, **kwgs):
             if data is None:
                 data = self.default_spooler(), self.context.root.active
             spooler, device_name = data
@@ -461,3 +486,106 @@ class Spoolers(Modifier):
 
         for i in range(5):
             self.get_or_make_spooler(str(i))
+
+        @context.console_command(
+            "test_dot_and_home",
+            input_type=("spooler", None),
+            hidden=True,
+        )
+        def run_home_and_dot_test(data, **kwgs):
+            if data is None:
+                data = self.default_spooler(), self.context.root.active
+            spooler, device_name = data
+
+            def home_dot_test():
+                for i in range(25):
+                    yield COMMAND_SET_ABSOLUTE
+                    yield COMMAND_MODE_RAPID
+                    yield COMMAND_HOME
+                    yield COMMAND_LASER_OFF
+                    yield COMMAND_WAIT_FINISH
+                    yield COMMAND_MOVE, 3000, 3000
+                    yield COMMAND_WAIT_FINISH
+                    yield COMMAND_LASER_ON
+                    yield COMMAND_WAIT, 0.05
+                    yield COMMAND_LASER_OFF
+                    yield COMMAND_WAIT_FINISH
+                yield COMMAND_HOME
+                yield COMMAND_WAIT_FINISH
+
+            spooler.job(home_dot_test)
+
+        @context.console_argument("transition_type", type=str)
+        @context.console_command(
+            "test_jog_transition",
+            help="test_jog_transition <finish,jog,switch>",
+            hidden=True,
+        )
+        def run_jog_transition_test(data, transition_type, **kwgs):
+            """ "
+            The Jog Transition Test is intended to test the jogging
+            """
+            if transition_type == "jog":
+                command = COMMAND_JOG
+            elif transition_type == "finish":
+                command = COMMAND_JOG_FINISH
+            elif transition_type == "switch":
+                command = COMMAND_JOG_SWITCH
+            else:
+                raise SyntaxError
+            if data is None:
+                data = self.default_spooler(), self.context.root.active
+            spooler, device_name = data
+
+            def jog_transition_test():
+                yield COMMAND_SET_ABSOLUTE
+                yield COMMAND_MODE_RAPID
+                yield COMMAND_HOME
+                yield COMMAND_LASER_OFF
+                yield COMMAND_WAIT_FINISH
+                yield COMMAND_MOVE, 3000, 3000
+                yield COMMAND_WAIT_FINISH
+                yield COMMAND_LASER_ON
+                yield COMMAND_WAIT, 0.05
+                yield COMMAND_LASER_OFF
+                yield COMMAND_WAIT_FINISH
+
+                yield COMMAND_SET_SPEED, 10.0
+
+                def pos(i):
+                    if i < 3:
+                        x = 200
+                    elif i < 6:
+                        x = -200
+                    else:
+                        x = 0
+                    if i % 3 == 0:
+                        y = 200
+                    elif i % 3 == 1:
+                        y = -200
+                    else:
+                        y = 0
+                    return x, y
+
+                for q in range(8):
+                    top = q & 1
+                    left = q & 2
+                    x_val = q & 3
+                    yield COMMAND_SET_DIRECTION, top, left, x_val, not x_val
+                    yield COMMAND_MODE_PROGRAM
+                    for j in range(9):
+                        jx, jy = pos(j)
+                        for k in range(9):
+                            kx, ky = pos(k)
+                            yield COMMAND_MOVE, 3000, 3000
+                            yield COMMAND_MOVE, 3000 + jx, 3000 + jy
+                            yield command, 3000 + jx + kx, 3000 + jy + ky
+                    yield COMMAND_MOVE, 3000, 3000
+                    yield COMMAND_MODE_RAPID
+                    yield COMMAND_WAIT_FINISH
+                    yield COMMAND_LASER_ON
+                    yield COMMAND_WAIT, 0.05
+                    yield COMMAND_LASER_OFF
+                    yield COMMAND_WAIT_FINISH
+
+            spooler.job(jog_transition_test)
