@@ -8,8 +8,11 @@ from ..basedevice import (
     PLOT_DIRECTION,
     PLOT_FINISH,
     PLOT_JOG,
+    PLOT_LEFT_UPPER,
     PLOT_RAPID,
+    PLOT_RIGHT_LOWER,
     PLOT_SETTING,
+    PLOT_START,
 )
 from ..lasercommandconstants import *
 
@@ -171,8 +174,11 @@ class GRBLDriver(Driver):
         line.append("X%f" % (x / self.scale))
         line.append("Y%f" % (y / self.scale))
         if self.power_updated:
-            line.append("S%f" % self.settings.power)
+            if self.settings.power is not None:
+                line.append("S%f" % self.settings.power)
             self.power_updated = False
+        if self.settings.speed is None:
+            self.settings.speed = 20.0
         if self.speed_updated:
             line.append("F%d" % int(self.feed_convert(self.settings.speed)))
             self.speed_updated = False
@@ -197,31 +203,29 @@ class GRBLDriver(Driver):
                 except TypeError:
                     break
                 on = int(on)
-                if on & PLOT_FINISH:  # Plot planner is ending.
-                    self.ensure_rapid_mode()
-                    continue
-                if on & PLOT_SETTING:  # Plot planner settings have changed.
-                    p_set = self.plot_planner.settings
-                    s_set = self.settings
-                    if p_set.power != s_set.power:
-                        self.set_power(p_set.power)
-                    if (
-                        p_set.speed != s_set.speed
-                        or p_set.raster_step != s_set.raster_step
-                    ):
-                        self.set_speed(p_set.speed)
-                        self.set_step(p_set.raster_step)
+                if on > 1:
+                    # Special Command.
+                    if on & PLOT_FINISH:  # Plot planner is ending.
+                        self.ensure_rapid_mode()
+                        break
+                    elif on & PLOT_SETTING:  # Plot planner settings have changed.
+                        p_set = self.plot_planner.settings
+                        s_set = self.settings
+                        if p_set.power != s_set.power:
+                            self.set_power(p_set.power)
+                        if (
+                            p_set.speed != s_set.speed
+                            or p_set.raster_step != s_set.raster_step
+                        ):
+                            self.set_speed(p_set.speed)
+                            self.set_step(p_set.raster_step)
+                            self.ensure_rapid_mode()
+                        self.settings.set_values(p_set)
+                    elif on & (
+                        PLOT_RAPID | PLOT_JOG
+                    ):  # Plot planner requests position change.
                         self.ensure_rapid_mode()
-                    self.settings.set_values(p_set)
                     continue
-                if on & PLOT_AXIS:  # Major Axis.
-                    continue
-                if on & PLOT_DIRECTION:
-                    continue
-                if on & (
-                    PLOT_RAPID | PLOT_JOG
-                ):  # Plot planner requests position change.
-                    self.ensure_rapid_mode()
                 if on == 0:
                     self.laser_on()
                 else:
@@ -792,7 +796,7 @@ class GRBLEmulator(Module):
         if data.startswith("cat"):
             return 2
         commands = {}
-        for c in self._tokenize_code(data):
+        for c in _tokenize_code(data):
             g = c[0]
             if g not in commands:
                 commands[g] = []
