@@ -14,7 +14,7 @@ class LaserSpeed:
     All values relate to a value in the counter to count off the number of oscillations within the
     (typically 22.1184) Mhz crystal. The max value here is 65535, with the addition of a diagonal delay.
 
-    For the M2 board, the original Chinese Software gave a slope of 12120. However experiments with the actual
+    For the M2 board, the original Chinese Software gave a slope of 12120. However, experiments with the actual
     physical speed put this value at 11142, which properly reflects that all speeds tend to be at 91.98% of the
     requested speed.
 
@@ -230,7 +230,7 @@ class LaserSpeed:
             # We do not need the diagonal code.
             if raster_step == 0:
                 if suffix_c:
-                    return "CV%s1C" % (encoded_speed)
+                    return "CV%s1C" % encoded_speed
                 else:
                     return "CV%s%1d" % (encoded_speed, acceleration)
         else:
@@ -316,7 +316,7 @@ class LaserSpeed:
         step_value = 0
         diagonal = 0
         if (end + 1) - start >= 9:
-            step_value = int(speed_code[start : start + 4])
+            step_value = int(speed_code[start : start + 3])
             diagonal = LaserSpeed.decode_16bit(speed_code[start + 3 : end])
         return code_value, accel, step_value, diagonal, raster_step, suffix_c
 
