@@ -1,211 +1,36 @@
+import math
+
 import wx
 
 from ..core.elements import LaserOperation
-from ..svgelements import Length
+from ..svgelements import Length, Group
 from .icons import icons8_laser_beam_52
 from .mwindow import MWindow
+from .propertiespanel import PropertiesPanel
 
 _ = wx.GetTranslation
 
 MILS_PER_MM = 39.3701
 
 
-class ExecuteJob(MWindow):
-    def __init__(self, *args, **kwds):
-        super().__init__(496, 573, *args, **kwds)
-        if len(args) >= 4:
-            plan_name = args[3]
-        else:
-            plan_name = 0
+class PlannerPanel(wx.Panel):
+    def __init__(self, *args, context=None, plan_name=None, **kwargs):
+        # begin wxGlade: ConsolePanel.__init__
+        kwargs["style"] = kwargs.get("style", 0) | wx.TAB_TRAVERSAL
+        wx.Panel.__init__(self, *args, **kwargs)
+        self.context = context
+
         self.plan_name = plan_name
 
-        # ==========
-        # MENU BAR
-        # ==========
-        self.preview_menu = wx.MenuBar()
-        wx_menu = wx.Menu()
-        wxglade_tmp_menu_sub = wx.Menu()
-
-        # ==========
-        # AUTO/BEFORE MENU
-        # ==========
-        self.preview_menu.menu_prehome = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Home"),
-            _("Automatically add a home command before all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_home_before,
-            id=self.preview_menu.menu_prehome.GetId(),
-        )
-        self.preview_menu.menu_prephysicalhome = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Physical Home"),
-            _("Automatically add a physical home command before all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_physicalhome_before,
-            id=self.preview_menu.menu_prephysicalhome.GetId(),
-        )
-
-        wx_menu.Append(wx.ID_ANY, _("Before"), wxglade_tmp_menu_sub, "")
-        wxglade_tmp_menu_sub = wx.Menu()
-
-        # ==========
-        # AUTO/AFTER MENU
-        # ==========
-        self.preview_menu.menu_autohome = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Home"),
-            _("Automatically add a home command after all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_home_after,
-            id=self.preview_menu.menu_autohome.GetId(),
-        )
-        self.preview_menu.menu_autophysicalhome = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Physical Home"),
-            _("Automatically add a physical home command after all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_physicalhome_after,
-            id=self.preview_menu.menu_autophysicalhome.GetId(),
-        )
-        self.preview_menu.menu_autoorigin = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Return to Origin"),
-            _("Automatically return to origin after a job"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_origin_after,
-            id=self.preview_menu.menu_autoorigin.GetId(),
-        )
-        self.preview_menu.menu_autounlock = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Unlock"),
-            _("Automatically unlock the rail after all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_unlock_after,
-            id=self.preview_menu.menu_autounlock.GetId(),
-        )
-        self.preview_menu.menu_autobeep = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Beep"),
-            _("Automatically add a beep after all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_beep_after,
-            id=self.preview_menu.menu_autobeep.GetId(),
-        )
-        self.preview_menu.menu_autointerrupt = wxglade_tmp_menu_sub.Append(
-            wx.ID_ANY,
-            _("Interrupt"),
-            _("Automatically add an interrupt after all jobs"),
-            wx.ITEM_CHECK,
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.on_check_interrupt_after,
-            id=self.preview_menu.menu_autointerrupt.GetId(),
-        )
-
-        wx_menu.Append(wx.ID_ANY, _("After"), wxglade_tmp_menu_sub, "")
-        self.preview_menu.Append(wx_menu, _("Automatic"))
-        wx_menu = wx.Menu()
-
-        # ==========
-        # ADD MENU
-        # ==========
-        self.preview_menu.menu_jobadd_home = wx_menu.Append(
-            wx.ID_ANY, _("Home"), _("Add a home")
-        )
-        self.Bind(
-            wx.EVT_MENU, self.jobadd_home, id=self.preview_menu.menu_jobadd_home.GetId()
-        )
-        self.preview_menu.menu_jobadd_autophysicalhome = wx_menu.Append(
-            wx.ID_ANY, _("Physical Home"), _("Add a physicalhome")
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.jobadd_physicalhome,
-            id=self.preview_menu.menu_jobadd_autophysicalhome.GetId(),
-        )
-        self.preview_menu.menu_jobadd_wait = wx_menu.Append(
-            wx.ID_ANY, _("Wait"), _("Add a wait")
-        )
-        self.Bind(
-            wx.EVT_MENU, self.jobadd_wait, id=self.preview_menu.menu_jobadd_wait.GetId()
-        )
-        self.preview_menu.menu_jobadd_beep = wx_menu.Append(
-            wx.ID_ANY, _("Beep"), _("Add a beep")
-        )
-        self.Bind(
-            wx.EVT_MENU, self.jobadd_beep, id=self.preview_menu.menu_jobadd_beep.GetId()
-        )
-        self.preview_menu.menu_jobadd_interrupt = wx_menu.Append(
-            wx.ID_ANY, _("Interrupt"), _("Add an interrupt")
-        )
-        self.Bind(
-            wx.EVT_MENU,
-            self.jobadd_interrupt,
-            id=self.preview_menu.menu_jobadd_interrupt.GetId(),
-        )
-
-        self.preview_menu.Append(wx_menu, _("Add"))
-
-        # ==========
-        # Tools Menu
-        # ==========
-        self.context.setting(bool, 'developer_mode', False)
-        if self.context.developer_mode:
-            wx_menu = wx.Menu()
-            self.preview_menu.Append(wx_menu, _("Tools"))
-
-            self.preview_menu.menu_send_back = wx_menu.Append(
-                wx.ID_ANY, _("Return to Operations"), _("Return the current Plan to Operations")
-            )
-            self.Bind(
-                wx.EVT_MENU,
-                self.jobchange_return_to_operations,
-                id=self.preview_menu.menu_send_back.GetId(),
-            )
-
-            self.preview_menu.menu_step_repeat = wx_menu.Append(
-                wx.ID_ANY, _("Step Repeat"), _("Execute Step Repeat")
-            )
-            self.Bind(
-                wx.EVT_MENU,
-                self.jobchange_step_repeat,
-                id=self.preview_menu.menu_step_repeat.GetId(),
-            )
-
-        self.SetMenuBar(self.preview_menu)
-        # ==========
-        # MENUBAR END
-        # ==========
-
         self.available_devices = [
             self.context.registered[i] for i in self.context.match("device")
         ]
         selected_spooler = self.context.root.active
         spools = [str(i) for i in self.context.match("device", suffix=True)]
-        index = spools.index(selected_spooler)
+        try:
+            index = spools.index(selected_spooler)
+        except ValueError:
+            index = 0
         self.connected_name = spools[index]
         self.connected_spooler, self.connected_driver, self.connected_output = (
             None,
@@ -232,27 +57,10 @@ class ExecuteJob(MWindow):
         self.list_command = wx.ListBox(self, wx.ID_ANY, choices=[])
 
         self.panel_operation = wx.Panel(self, wx.ID_ANY)
-
-        self.check_rapid_moves_between = wx.CheckBox(
-            self, wx.ID_ANY, _("Rapid Moves Between Objects")
+        choices = self.context.registered["choices/optimize"][:7]
+        self.panel_optimize = PropertiesPanel(
+            self, wx.ID_ANY, context=self.context, choices=choices
         )
-        self.check_reduce_travel_time = wx.CheckBox(
-            self, wx.ID_ANY, _("Reduce Travel Time")
-        )
-
-        self.check_merge_passes = wx.CheckBox(
-            self, wx.ID_ANY, _("Merge Passes")
-        )
-        self.check_merge_ops = wx.CheckBox(
-            self, wx.ID_ANY, _("Merge Operations")
-        )
-        self.check_cut_inner_first = wx.CheckBox(self, wx.ID_ANY, _("Cut Inner First"))
-        # self.check_reduce_direction_changes = wx.CheckBox(
-        #     self, wx.ID_ANY, _("Reduce Direction Changes")
-        # )
-        # self.check_remove_overlap_cuts = wx.CheckBox(
-        #     self, wx.ID_ANY, _("Remove Overlap Cuts")
-        # )
         self.button_start = wx.Button(self, wx.ID_ANY, _("Start"))
 
         self.__set_properties()
@@ -269,84 +77,17 @@ class ExecuteJob(MWindow):
         self.Bind(
             wx.EVT_LISTBOX_DCLICK, self.on_listbox_commands_dclick, self.list_command
         )
-        self.Bind(
-            wx.EVT_CHECKBOX, self.on_check_merge_passes, self.check_merge_passes
-        )
-        self.Bind(
-            wx.EVT_CHECKBOX, self.on_check_merge_ops, self.check_merge_ops
-        )
-        self.Bind(
-            wx.EVT_CHECKBOX, self.on_check_rapid_between, self.check_rapid_moves_between
-        )
-        self.Bind(
-            wx.EVT_CHECKBOX, self.on_check_reduce_travel, self.check_reduce_travel_time
-        )
-        self.Bind(
-            wx.EVT_CHECKBOX, self.on_check_inner_first, self.check_cut_inner_first
-        )
-        # self.Bind(
-        #     wx.EVT_CHECKBOX,
-        #     self.on_check_reduce_directions,
-        #     self.check_reduce_direction_changes,
-        # )
-        # self.Bind(
-        #     wx.EVT_CHECKBOX,
-        #     self.on_check_remove_overlap,
-        #     self.check_remove_overlap_cuts,
-        # )
-
         self.Bind(wx.EVT_BUTTON, self.on_button_start, self.button_start)
-        # end wxGlade
         self.stage = 0
 
     def __set_properties(self):
         # begin wxGlade: Preview.__set_properties
-        _icon = wx.NullIcon
-        _icon.CopyFromBitmap(icons8_laser_beam_52.GetBitmap())
-        self.SetIcon(_icon)
-        self.SetTitle(_("Execute Job"))
+
         self.combo_device.SetToolTip(
             _("Select the device to which to send the current job")
         )
         self.list_operations.SetToolTip(_("Operations being added to the current job"))
         self.list_command.SetToolTip(_("Commands being applied to the current job"))
-        # self.slider_progress.SetToolTip(_("Preview slider to set progress position"))
-        # self.text_operation_name.SetToolTip(_("Current Operation Being Processed"))
-        # self.gauge_operation.SetToolTip(_("Gauge of Operation Progress"))
-        # self.text_time_laser.SetToolTip(_("Time Estimate: Lasering Time"))
-        # self.text_time_travel.SetToolTip(_("Time Estimate: Traveling Time"))
-        # self.text_time_total.SetToolTip(_("Time Estimate: Total Time"))
-        self.check_merge_passes.SetToolTip(
-            _(
-                "Combine passes into the same optimization"
-            )
-        )
-        self.check_merge_ops.SetToolTip(
-            _(
-                "Combine operations into the same optimization"
-            )
-        )
-        self.check_rapid_moves_between.SetToolTip(
-            _(
-                "Travel between objects (laser off) at the default/rapid speed rather than at the current laser-on speed"
-            )
-        )
-        self.check_reduce_travel_time.SetToolTip(
-            _("Reduce the travel time by optimizing the order of the elements")
-        )
-        self.check_cut_inner_first.SetToolTip(
-            _(
-                "Ensure that inside burns are done before an outside cut which might result in the cut piece shifting or dropping out of the material, while still requiring additonal cuts."
-            )
-        )
-        # self.check_reduce_direction_changes.SetToolTip(
-        #     _("Reorder to reduce the number of sharp directional changes")
-        # )
-        # self.check_reduce_direction_changes.Hide()
-        # self.check_remove_overlap_cuts.SetToolTip(
-        #     _("Remove elements of overlapped cuts")
-        # )
-        # self.check_remove_overlap_cuts.Hide()
         self.button_start.SetBackgroundColour(wx.Colour(0, 255, 0))
         self.button_start.SetFont(
             wx.Font(
@@ -369,41 +110,15 @@ class ExecuteJob(MWindow):
         sizer_optimizations = wx.StaticBoxSizer(
             wx.StaticBox(self, wx.ID_ANY, _("Optimizations")), wx.VERTICAL
         )
-        # sizer_time = wx.BoxSizer(wx.HORIZONTAL)
-        # sizer_total_time = wx.StaticBoxSizer(
-        #     wx.StaticBox(self, wx.ID_ANY, "Total Time"), wx.VERTICAL
-        # )
-        # sizer_travel_time = wx.StaticBoxSizer(
-        #     wx.StaticBox(self, wx.ID_ANY, "Travel Time"), wx.VERTICAL
-        # )
-        # sizer_laser_time = wx.StaticBoxSizer(
-        #     wx.StaticBox(self, wx.ID_ANY, "Laser Time"), wx.VERTICAL
-        # )
-        # sizer_operation = wx.BoxSizer(wx.HORIZONTAL)
         sizer_main = wx.BoxSizer(wx.HORIZONTAL)
         sizer_frame.Add(self.combo_device, 0, wx.EXPAND, 0)
         sizer_main.Add(self.list_operations, 2, wx.EXPAND, 0)
         sizer_main.Add(self.list_command, 2, wx.EXPAND, 0)
         sizer_frame.Add(sizer_main, 1, wx.EXPAND, 0)
-        # sizer_frame.Add(self.slider_progress, 0, wx.EXPAND, 0)
-        # sizer_operation.Add(self.text_operation_name, 2, 0, 0)
-        # sizer_operation.Add(self.gauge_operation, 7, wx.EXPAND, 0)
-        # self.panel_operation.SetSizer(sizer_operation)
+
         sizer_frame.Add(self.panel_operation, 0, wx.EXPAND, 0)
-        # sizer_laser_time.Add(self.text_time_laser, 0, wx.EXPAND, 0)
-        # sizer_time.Add(sizer_laser_time, 1, wx.EXPAND, 0)
-        # sizer_travel_time.Add(self.text_time_travel, 0, wx.EXPAND, 0)
-        # sizer_time.Add(sizer_travel_time, 1, wx.EXPAND, 0)
-        # sizer_total_time.Add(self.text_time_total, 0, wx.EXPAND, 0)
-        # sizer_time.Add(sizer_total_time, 1, wx.EXPAND, 0)
-        # sizer_frame.Add(sizer_time, 0, wx.EXPAND, 0)
-        sizer_optimizations.Add(self.check_rapid_moves_between, 0, 0, 0)
-        sizer_optimizations.Add(self.check_reduce_travel_time, 0, 0, 0)
-        sizer_optimizations.Add(self.check_merge_ops, 0, 0, 0)
-        sizer_optimizations.Add(self.check_merge_passes, 0, 0, 0)
-        sizer_optimizations.Add(self.check_cut_inner_first, 0, 0, 0)
-        # sizer_optimizations.Add(self.check_reduce_direction_changes, 0, 0, 0)
-        # sizer_optimizations.Add(self.check_remove_overlap_cuts, 0, 0, 0)
+
+        sizer_optimizations.Add(self.panel_optimize)
         sizer_options.Add(sizer_optimizations, 2, wx.EXPAND, 0)
         sizer_options.Add(self.button_start, 3, wx.EXPAND, 0)
         sizer_frame.Add(sizer_options, 0, wx.EXPAND, 0)
@@ -411,65 +126,6 @@ class ExecuteJob(MWindow):
         self.Layout()
         # end wxGlade
 
-    def on_check_home_before(self, event=None):  # wxGlade: JobInfo.<event_handler>
-        self.context.prehome = self.preview_menu.menu_prehome.IsChecked()
-
-    def on_check_home_after(self, event=None):  # wxGlade: JobInfo.<event_handler>
-        self.context.autohome = self.preview_menu.menu_autohome.IsChecked()
-
-    def on_check_physicalhome_before(
-        self, event=None
-    ):  # wxGlade: JobInfo.<event_handler>
-        self.context.prephysicalhome = (
-            self.preview_menu.menu_prephysicalhome.IsChecked()
-        )
-
-    def on_check_physicalhome_after(
-        self, event=None
-    ):  # wxGlade: JobInfo.<event_handler>
-        self.context.autophysicalhome = (
-            self.preview_menu.menu_autophysicalhome.IsChecked()
-        )
-
-    def on_check_origin_after(self, event=None):  # wxGlade: JobInfo.<event_handler>
-        self.context.autoorigin = self.preview_menu.menu_autoorigin.IsChecked()
-
-    def on_check_beep_after(self, event=None):  # wxGlade: JobInfo.<event_handler>
-        self.context.autobeep = self.preview_menu.menu_autobeep.IsChecked()
-
-    def on_check_interrupt_after(self, event=None):  # wxGlade: Preview.<event_handler>
-        self.context.autointerrupt = self.preview_menu.menu_autointerrupt.IsChecked()
-
-    def on_check_unlock_after(self, event=None):  # wxGlade: Preview.<event_handler>
-        self.context.postunlock = self.preview_menu.menu_autounlock.IsChecked()
-
-    def on_check_reduce_travel(self, event=None):  # wxGlade: Preview.<event_handler>
-        self.context.opt_reduce_travel = self.check_reduce_travel_time.IsChecked()
-        self.check_merge_ops.Enable(self.context.opt_reduce_travel)
-        self.check_merge_passes.Enable(self.context.opt_reduce_travel)
-
-    def on_check_inner_first(self, event=None):  # wxGlade: Preview.<event_handler>
-        self.context.opt_inner_first = self.check_cut_inner_first.IsChecked()
-
-    # def on_check_reduce_directions(
-    #     self, event=None
-    # ):  # wxGlade: Preview.<event_handler>
-    #     self.context.opt_reduce_directions = (
-    #         self.check_reduce_direction_changes.IsChecked()
-    #     )
-    #
-    # def on_check_remove_overlap(self, event=None):  # wxGlade: Preview.<event_handler>
-    #     self.context.opt_remove_overlap = self.check_remove_overlap_cuts.IsChecked()
-
-    def on_check_merge_ops(self, event=None):
-        self.context.opt_merge_ops = self.check_merge_ops.IsChecked()
-
-    def on_check_merge_passes(self, event=None):
-        self.context.opt_merge_passes = self.check_merge_passes.IsChecked()
-
-    def on_check_rapid_between(self, event=None):  # wxGlade: Preview.<event_handler>
-        self.context.opt_rapid_between = self.check_rapid_moves_between.IsChecked()
-
     def jobchange_return_to_operations(self, event=None):
         self.context("plan%s return clear\n" % (self.plan_name))
 
@@ -477,7 +133,6 @@ class ExecuteJob(MWindow):
         dlg = wx.TextEntryDialog(
             self, _("How many copies wide?"), _("Enter Columns"), ""
         )
-        dlg.SetValue("5")
 
         if dlg.ShowModal() == wx.ID_OK:
             try:
@@ -491,7 +146,6 @@ class ExecuteJob(MWindow):
         dlg.Destroy()
 
         dlg = wx.TextEntryDialog(self, _("How many copies high?"), _("Enter Rows"), "")
-        dlg.SetValue("5")
         if dlg.ShowModal() == wx.ID_OK:
             try:
                 rows = int(dlg.GetValue())
@@ -503,20 +157,48 @@ class ExecuteJob(MWindow):
             return
         dlg.Destroy()
 
+        elems = []
+        cutplan = self.context.default_plan()
+        for node in cutplan.plan:
+            if type(node) is LaserOperation:
+                objs = [e.object for e in node.children]
+                elems.extend(objs)
+        bounds = Group.union_bbox(elems)
+
+        try:
+            width = math.ceil(bounds[2] - bounds[0])
+            height = math.ceil(bounds[3] - bounds[1])
+        except TypeError:
+            width = None
+            height = None
+
         dlg = wx.TextEntryDialog(
             self,
-            _("How far apart are these copies width-wise? eg. 2in, 3cm, 50mm, 10%"),
-            _("Enter X Gap"),
+            _("How far apart are these copies width-wise? eg. 2in, 3cm, 50mm, 10%")
+            + "\n\n"
+            + _("This should be the item width + any gap."),
+            _("Enter X Delta"),
             "",
         )
-        dlg.SetValue("")
+
+        if self.context:
+            conversion = self.context.units_convert
+            name = self.context.units_name
+            if height:
+                height = "%.1f%s" % (height / conversion, name)
+            if width:
+                width  = "%.1f%s" % (width / conversion, name)
+        dlg.SetValue(str(width) if width is not None else "%f%%" % (100.0 / cols))
         bed_dim = self.context.root
         bed_dim.setting(int, "bed_width", 310)
         bed_dim.setting(int, "bed_height", 210)
         if dlg.ShowModal() == wx.ID_OK:
             try:
                 x_distance = Length(dlg.GetValue()).value(
-                    ppi=1000.0, relative_length=bed_dim.bed_width * MILS_PER_MM
+                    ppi=1000.0,
+                    relative_length=width
+                    if width is not None
+                    else bed_dim.bed_width * MILS_PER_MM,
                 )
             except ValueError:
                 dlg.Destroy()
@@ -531,15 +213,20 @@ class ExecuteJob(MWindow):
 
         dlg = wx.TextEntryDialog(
             self,
-            _("How far apart are these copies height-wise? eg. 2in, 3cm, 50mm, 10%"),
-            _("Enter Y Gap"),
+            _("How far apart are these copies height-wise? eg. 2in, 3cm, 50mm, 10%")
+            + "\n\n"
+            + _("This should be the item height + any gap."),
+            _("Enter Y Delta"),
             "",
         )
-        dlg.SetValue("")
+        dlg.SetValue(str(height) if height is not None else "%f%%" % (100.0 / rows))
         if dlg.ShowModal() == wx.ID_OK:
             try:
                 y_distance = Length(dlg.GetValue()).value(
-                    ppi=1000.0, relative_length=bed_dim.bed_width * MILS_PER_MM
+                    ppi=1000.0,
+                    relative_length=height
+                    if height is not None
+                    else bed_dim.bed_height * MILS_PER_MM,
                 )
             except ValueError:
                 dlg.Destroy()
@@ -647,28 +334,24 @@ class ExecuteJob(MWindow):
                 self.context("plan%s spool%s\n" % (self.plan_name, self.connected_name))
                 if self.context.auto_spooler:
                     self.context("window open JobSpooler\n")
-                self.Close()
+                try:
+                    self.GetParent().Close()
+                except (TypeError, AttributeError):
+                    pass
         self.update_gui()
 
-    def window_open(self):
+    def initialize(self):
         rotary_context = self.context.get_context("rotary/1")
         rotary_context.setting(bool, "rotary", False)
         rotary_context.setting(float, "scale_x", 1.0)
         rotary_context.setting(float, "scale_y", 1.0)
-        self.context.setting(bool, "auto_spooler", True)
-        self.context.setting(bool, "prehome", False)
-        self.context.setting(bool, "autohome", False)
-        self.context.setting(bool, "prephysicalhome", False)
-        self.context.setting(bool, "autophysicalhome", False)
-        self.context.setting(bool, "autoorigin", False)
-        self.context.setting(bool, "autobeep", True)
-        self.context.setting(bool, "autointerrupt", False)
-        self.context.setting(bool, "postunlock", False)
         self.context.setting(int, "opt_closed_distance", 15)
         self.context.setting(bool, "opt_merge_passes", False)
         self.context.setting(bool, "opt_merge_ops", False)
         self.context.setting(bool, "opt_reduce_travel", True)
+        self.context.setting(bool, "opt_complete_subpaths", False)
         self.context.setting(bool, "opt_inner_first", True)
+        self.context.setting(bool, "opt_inners_grouped", False)
         self.context.setting(bool, "opt_reduce_directions", False)
         self.context.setting(bool, "opt_remove_overlap", False)
         self.context.setting(bool, "opt_rapid_between", True)
@@ -677,36 +360,25 @@ class ExecuteJob(MWindow):
 
         self.context.listen("element_property_reload", self.on_element_property_update)
         self.context.listen("plan", self.plan_update)
-
-        self.preview_menu.menu_prehome.Check(bool(self.context.prehome))
-        self.preview_menu.menu_autohome.Check(bool(self.context.autohome))
-        self.preview_menu.menu_prephysicalhome.Check(bool(self.context.prephysicalhome))
-        self.preview_menu.menu_autophysicalhome.Check(
-            bool(self.context.autophysicalhome)
-        )
-
-        self.preview_menu.menu_autoorigin.Check(bool(self.context.autoorigin))
-        self.preview_menu.menu_autobeep.Check(bool(self.context.autobeep))
-        self.preview_menu.menu_autointerrupt.Check(bool(self.context.autointerrupt))
-        self.preview_menu.menu_autounlock.Check(bool(self.context.postunlock))
-
-        self.check_rapid_moves_between.SetValue(self.context.opt_rapid_between)
-        self.check_reduce_travel_time.SetValue(self.context.opt_reduce_travel)
-        self.check_merge_passes.SetValue(self.context.opt_merge_passes)
-        self.check_merge_ops.SetValue(self.context.opt_merge_ops)
-        self.check_merge_ops.Enable(self.context.opt_reduce_travel)
-        self.check_merge_passes.Enable(self.context.opt_reduce_travel)
-        self.check_cut_inner_first.SetValue(self.context.opt_inner_first)
+        #
+        # self.check_rapid_moves_between.SetValue(self.context.opt_rapid_between)
+        # self.check_reduce_travel_time.SetValue(self.context.opt_reduce_travel)
+        # self.check_merge_passes.SetValue(self.context.opt_merge_passes)
+        # self.check_merge_ops.SetValue(self.context.opt_merge_ops)
+        # self.check_merge_ops.Enable(self.context.opt_reduce_travel)
+        # self.check_merge_passes.Enable(self.context.opt_reduce_travel)
+        # self.check_cut_inner_first.SetValue(self.context.opt_inner_first)
         # self.check_reduce_direction_changes.SetValue(self.context.opt_reduce_directions)
         # self.check_remove_overlap_cuts.SetValue(self.context.opt_remove_overlap)
 
         cutplan = self.context.default_plan()
+        self.Children[0].SetFocus()
         if len(cutplan.plan) == 0 and len(cutplan.commands) == 0:
             self.context("plan%s copy preprocess\n" % self.plan_name)
 
         self.update_gui()
 
-    def window_close(self):
+    def finalize(self):
         self.context("plan%s clear\n" % self.plan_name)
 
         self.context.unlisten(
@@ -773,3 +445,115 @@ class ExecuteJob(MWindow):
             self.button_start.SetBackgroundColour(wx.Colour(255, 255, 255))
             self.button_start.SetToolTip(_("Send this data to the spooler"))
         self.Refresh()
+
+
+class ExecuteJob(MWindow):
+    def __init__(self, *args, **kwds):
+        super().__init__(496, 573, *args, **kwds)
+
+        if len(args) >= 4:
+            plan_name = args[3]
+        else:
+            plan_name = 0
+        self.panel = PlannerPanel(
+            self, wx.ID_ANY, context=self.context, plan_name=plan_name
+        )
+        self.panel.Bind(wx.EVT_RIGHT_DOWN, self.on_menu, self.panel)
+        self.panel.list_operations.Bind(wx.EVT_RIGHT_DOWN, self.on_menu, self.panel.list_operations)
+        self.panel.list_command.Bind(wx.EVT_RIGHT_DOWN, self.on_menu, self.panel.list_command)
+        _icon = wx.NullIcon
+        _icon.CopyFromBitmap(icons8_laser_beam_52.GetBitmap())
+        self.SetIcon(_icon)
+        self.SetTitle(_("Execute Job"))
+
+        # ==========
+        # MENU BAR
+        # ==========
+        from platform import system as _sys
+
+        if _sys() != "Darwin":
+            self.preview_menu = wx.MenuBar()
+            self.create_menu(self.preview_menu.Append)
+            self.SetMenuBar(self.preview_menu)
+        # ==========
+        # MENUBAR END
+        # ==========
+
+    def on_menu(self, event):
+        from .wxutils import create_menu_for_choices
+
+        menu = create_menu_for_choices(
+            self, self.context.registered["choices/planner"]
+        )
+        self.PopupMenu(menu)
+        menu.Destroy()
+
+    def create_menu(self, append):
+        from .wxutils import create_menu_for_choices
+
+        wx_menu = create_menu_for_choices(
+            self, self.context.registered["choices/planner"]
+        )
+        append(wx_menu, _("Automatic"))
+
+        # ==========
+        # ADD MENU
+        # ==========
+        wx_menu = wx.Menu()
+        append(wx_menu, _("Add"))
+
+        self.Bind(
+            wx.EVT_MENU,
+            self.panel.jobadd_home,
+            wx_menu.Append(wx.ID_ANY, _("Home"), _("Add a home")),
+        )
+        self.Bind(
+            wx.EVT_MENU,
+            self.panel.jobadd_physicalhome,
+            wx_menu.Append(wx.ID_ANY, _("Physical Home"), _("Add a physicalhome")),
+        )
+        self.Bind(
+            wx.EVT_MENU,
+            self.panel.jobadd_wait,
+            wx_menu.Append(wx.ID_ANY, _("Wait"), _("Add a wait")),
+        )
+        self.Bind(
+            wx.EVT_MENU,
+            self.panel.jobadd_beep,
+            wx_menu.Append(wx.ID_ANY, _("Beep"), _("Add a beep")),
+        )
+        self.Bind(
+            wx.EVT_MENU,
+            self.panel.jobadd_interrupt,
+            wx_menu.Append(wx.ID_ANY, _("Interrupt"), _("Add an interrupt")),
+        )
+
+        # ==========
+        # Tools Menu
+        # ==========
+        wx_menu = wx.Menu()
+        append(wx_menu, _("Tools"))
+
+        self.context.setting(bool, "developer_mode", False)
+        if self.context.developer_mode:
+            self.Bind(
+                wx.EVT_MENU,
+                self.panel.jobchange_return_to_operations,
+                wx_menu.Append(
+                    wx.ID_ANY,
+                    _("Return to Operations"),
+                    _("Return the current Plan to Operations"),
+                ),
+            )
+
+        self.Bind(
+            wx.EVT_MENU,
+            self.panel.jobchange_step_repeat,
+            wx_menu.Append(wx.ID_ANY, _("Step Repeat"), _("Execute Step Repeat")),
+        )
+
+    def window_open(self):
+        self.panel.initialize()
+
+    def window_close(self):
+        self.panel.finalize()
