@@ -20,6 +20,7 @@ from ..svgelements import (
     SVGImage,
     SVGText,
 )
+from .icons import icons8_image_50
 from .zmatrix import ZMatrix
 
 DRAW_MODE_FILLS = 0x000001
@@ -41,6 +42,7 @@ DRAW_MODE_TREE = 0x0080000
 DRAW_MODE_INVERT = 0x400000
 DRAW_MODE_FLIPXY = 0x800000
 DRAW_MODE_LINEWIDTH = 0x1000000
+DRAW_MODE_ALPHABLACK = 0x2000000  # Set means do not alphablack images
 
 
 def swizzlecolor(c):
@@ -154,7 +156,10 @@ class LaserRender:
             self.color.SetRGBA(swizzle_color | alpha << 24)  # wx has BBGGRR
             self.pen.SetColour(self.color)
             try:
-                self.pen.SetWidth(width)
+                try:
+                    self.pen.SetWidth(width)
+                except TypeError:
+                    self.pen.SetWidth(int(width))
             except OverflowError:
                 pass  # Exceeds 32 bit signed integer.
             gc.SetPen(self.pen)
@@ -237,12 +242,14 @@ class LaserRender:
                 )
             elif isinstance(cut, RasterCut):
                 image = cut.image
-                try:
-                    matrix = Matrix(image.transform)
-                except AttributeError:
-                    matrix = Matrix()
-                matrix.post_translate(x, y)
                 gc.PushState()
+                matrix = Matrix()
+                matrix.post_translate(x, y)  # Add cutcode offset.
+
+                matrix.post_scale(
+                    cut.step
+                )  # Scale up the image by the step for simulation
+                matrix.post_translate(cut.tx, cut.ty)  # Adjust image xy
                 if matrix is not None and not matrix.is_identity():
                     gc.ConcatTransform(
                         wx.GraphicsContext.CreateMatrix(gc, ZMatrix(matrix))
@@ -254,13 +261,23 @@ class LaserRender:
                     cache_id = cut.cache_id
                 except AttributeError:
                     pass
-                if cache_id != id(image.image):
+                if cache_id != id(image):
                     cache = None
                 if cache is None:
-                    cut.c_width, cut.c_height = image.image.size
-                    cut.cache = self.make_thumbnail(image.image, dewhite=True)
-                    cut.cache_id = id(image.image)
-                gc.DrawBitmap(cut.cache, 0, 0, cut.c_width, cut.c_height)
+                    cut.c_width, cut.c_height = image.size
+                    try:
+                        cut.cache = self.make_thumbnail(image, maximum=1000)
+                    except (MemoryError, RuntimeError):
+                        cut.cache = None
+                    cut.cache_id = id(image)
+                if cut.cache is not None:
+                    gc.DrawBitmap(cut.cache, 0, 0, cut.c_width, cut.c_height)
+                else:
+                    gc.SetBrush(wx.RED_BRUSH)
+                    gc.DrawRectangle(0, 0, cut.c_width, cut.c_height)
+                    gc.DrawBitmap(
+                        icons8_image_50.GetBitmap(), 0, 0, cut.c_width, cut.c_height
+                    )
                 gc.PopState()
             last_point = end
         if p is not None:
@@ -335,7 +352,10 @@ class LaserRender:
                         text.font_size, text.font_size, text.x, text.y
                     )
                 text.font_size = 1  # No zero sized fonts.
-            font = wx.Font(text.font_size, wx.SWISS, wx.NORMAL, wx.BOLD)
+            try:
+                font = wx.Font(text.font_size, wx.SWISS, wx.NORMAL, wx.BOLD)
+            except TypeError:
+                font = wx.Font(int(text.font_size), wx.SWISS, wx.NORMAL, wx.BOLD)
             try:
                 f = []
                 if text.font_family is not None:
@@ -397,11 +417,17 @@ class LaserRender:
                 except AttributeError:
                     max_allowed = 2048
                 node.c_width, node.c_height = image.image.size
-                node.cache = self.make_thumbnail(image.image, maximum=max_allowed)
+                node.cache = self.make_thumbnail(
+                    image.image,
+                    maximum=max_allowed,
+                    alphablack=draw_mode & DRAW_MODE_ALPHABLACK == 0,
+                )
             gc.DrawBitmap(node.cache, 0, 0, node.c_width, node.c_height)
         else:
             node.c_width, node.c_height = image.image.size
-            cache = self.make_thumbnail(image.image)
+            cache = self.make_thumbnail(
+                image.image, alphablack=draw_mode & DRAW_MODE_ALPHABLACK == 0
+            )
             gc.DrawBitmap(cache, 0, 0, node.c_width, node.c_height)
         gc.PopState()
 
@@ -410,7 +436,9 @@ class LaserRender:
     ):
         """
         Make Raster turns an iterable of elements and a bounds into an image of the designated size, taking into account
-        the step size etc.
+        the step size. The physical pixels in the image is reduced by the step size then the matrix for the element is
+        scaled up by the same amount. This makes step size work like inverse dpi and correctly sets the image scale to
+        the step scale for 1:1 sizes independent of the scale.
 
         This function requires both wxPython and Pillow.
 
@@ -442,10 +470,15 @@ class LaserRender:
             width = image_width
         if height is None:
             height = image_height
+        # Scale physical image down by step amount.
         width /= float(step)
         height /= float(step)
         width = int(width)
         height = int(height)
+        if width <= 0:
+            width = 1
+        if height <= 0:
+            height = 1
         bmp = wx.Bitmap(width, height, 32)
         dc = wx.MemoryDC()
         dc.SelectObject(bmp)
@@ -454,6 +487,8 @@ class LaserRender:
 
         matrix = Matrix()
         matrix.post_translate(-xmin, -ymin)
+
+        # Scale affine matrix up by step amount scaled down.
         scale_x = width / float(image_width)
         scale_y = height / float(image_height)
         scale = min(scale_x, scale_y)
@@ -482,7 +517,7 @@ class LaserRender:
         return image
 
     def make_thumbnail(
-        self, pil_data, maximum=None, width=None, height=None, dewhite=False
+        self, pil_data, maximum=None, width=None, height=None, alphablack=True
     ):
         """Resizes the given pil image into wx.Bitmap object that fits the constraints."""
         image_width, image_height = pil_data.size
@@ -500,20 +535,22 @@ class LaserRender:
             width = int(round(width * scale))
             height = int(round(height * scale))
         if image_width != width or image_height != height:
-            pil_data = pil_data.copy().resize((width, height))
+            pil_data = pil_data.resize((width, height))
         else:
             pil_data = pil_data.copy()
-        if dewhite:
-            try:
-                img = pil_data.convert("L")
-                black = Image.new("L", img.size, color="black")
-                img = img.point(lambda e: 255 - e)
-                black.putalpha(img)
-                pil_data = black
-            except MemoryError:
-                pass  # Not enough memory to do this.
+        if not alphablack:
+            return wx.Bitmap.FromBufferRGBA(
+                width, height, pil_data.convert("RGBA").tobytes()
+            )
 
-        if pil_data.mode != "RGBA":
-            pil_data = pil_data.convert("RGBA")
-        pil_bytes = pil_data.tobytes()
-        return wx.Bitmap.FromBufferRGBA(width, height, pil_bytes)
+        try:
+            # If transparent we paste 0 into the pil_data
+            mask = pil_data.getchannel("A").point(lambda e: 255 - e)
+            pil_data.paste(mask, None, mask)
+        except ValueError:
+            pass
+        if pil_data.mode != "L":
+            pil_data = pil_data.convert("L")
+        black = Image.new("RGBA", pil_data.size, "black")
+        black.putalpha(pil_data.point(lambda e: 255 - e))
+        return wx.Bitmap.FromBufferRGBA(width, height, black.tobytes())
