@@ -1,14 +1,9 @@
-from meerk40t.gui.scene.scene import Widget
-
-try:
-    from math import tau
-except ImportError:
-    from math import pi
-
-    tau = 2 * pi
-
+import math
 import wx
 
+from meerk40t.gui.scene.scene import Widget
+from meerk40t.gui.wxutils import create_menu
+
 from meerk40t.gui.laserrender import (
     DRAW_MODE_BACKGROUND,
     DRAW_MODE_GRID,
@@ -50,17 +45,23 @@ class ElementsWidget(Widget):
     The ElementsWidget is tasked with drawing the elements within the scene. It also
     server to process leftclick in order to emphasize the given object.
     """
-    def __init__(self, scene, root, renderer):
+
+    def __init__(self, scene, renderer):
         Widget.__init__(self, scene, all=True)
         self.renderer = renderer
-        self.root = root
 
     def hit(self):
         return HITCHAIN_HIT
 
     def process_draw(self, gc):
         context = self.scene.context
-        zoom_scale = 1 / self.scene.widget_root.scene_widget.matrix.value_scale_x()
+        matrix = self.scene.widget_root.scene_widget.matrix
+        scale_x = matrix.value_scale_x()
+        try:
+            zoom_scale = 1 / scale_x
+        except ZeroDivisionError:
+            matrix.reset()
+            zoom_scale = 1
         if zoom_scale < 1:
             zoom_scale = 1
         self.renderer.render(
@@ -74,7 +75,7 @@ class ElementsWidget(Widget):
         if event_type == "leftclick":
             elements = self.scene.context.elements
             elements.set_emphasized_by_position(space_pos)
-            self.root.select_in_tree_by_emphasis()
+            self.scene.context.signal("select_emphasized_tree", 0)
             return RESPONSE_CONSUME
         return RESPONSE_DROP
 
@@ -84,9 +85,9 @@ class SelectionWidget(Widget):
     Selection Widget it tasked with drawing the selection box and managing the events
     dealing with moving, resizing and altering the selected object.
     """
-    def __init__(self, scene, root):
+
+    def __init__(self, scene):
         Widget.__init__(self, scene, all=False)
-        self.root = root
         self.elements = scene.context.elements
         self.selection_pen = wx.Pen()
         self.selection_pen.SetColour(wx.Colour(0xA0, 0x7F, 0xA0))
@@ -105,8 +106,30 @@ class SelectionWidget(Widget):
             self.top = bounds[1]
             self.right = bounds[2]
             self.bottom = bounds[3]
+
+            # Adjust boundaries to ensure a minimum height / width so that
+            # dots and horizontal / vertical lines get a move icon
+            width = self.right - self.left
+            height = self.bottom - self.top
+            matrix = self.parent.matrix
+            # Twice size of equivalent in event:hover
+            try:
+                xmin = 10 / matrix.value_scale_x()
+                ymin = 10 / matrix.value_scale_y()
+            except ZeroDivisionError:
+                matrix.reset()
+                ymin = 10
+                xmin = 10
+            if width < xmin:
+                width = (xmin - width) / 2
+                self.left -= width
+                self.right += width
+            if height < ymin:
+                height = (ymin - height) / 2
+                self.top -= height
+                self.bottom += height
+
             self.clear()
-            self.scene.context.signal("refresh_scene", 0)
             return HITCHAIN_HIT
         else:
             self.left = float("inf")
@@ -114,74 +137,74 @@ class SelectionWidget(Widget):
             self.right = -float("inf")
             self.bottom = -float("inf")
             self.clear()
-            self.scene.context.signal("refresh_scene", 0)
             return HITCHAIN_DELEGATE
 
     def event(self, window_pos=None, space_pos=None, event_type=None):
         elements = self.elements
         if event_type == "hover_start":
-            self.cursor = wx.CURSOR_SIZING
-            self.scene.gui.SetCursor(wx.Cursor(self.cursor))
+            self.scene.cursor("sizing")
             return RESPONSE_CHAIN
-        if event_type == "hover_end":
-            self.cursor = wx.CURSOR_ARROW
-            self.scene.gui.SetCursor(wx.Cursor(self.cursor))
+        elif event_type == "hover_end" or event_type == "lost":
+            self.scene.cursor("arrow")
             return RESPONSE_CHAIN
-        if event_type == "hover":
+        elif event_type == "hover":
             matrix = self.parent.matrix
             xin = space_pos[0] - self.left
             yin = space_pos[1] - self.top
-            # TODO Handle distance should be constant regardless of zoom factor. May need to scale by screen DPI.
+            # Half size of equivalent in hit
             xmin = 5 / matrix.value_scale_x()
             ymin = 5 / matrix.value_scale_y()
             # Adjust sizing of hover border as follows:
             # 1. If object is very small so move area is smaller than 1/2 or even non-existent, prefer move to size by setting border to zero
-            # 2. Otherwise try to expand by up to 2 (to make it easier to hover) but never less than xmin and never expanded 
+            # 2. Otherwise try to expand by up to 2 (to make it easier to hover) but never less than xmin and never expanded
             #    to be more than 1/4 of the width or height
-            xmin = min(xmin * 2.0, max(self.width / 4.0, xmin)) if xmin <= self.width / 4.0 else 0.0
-            ymin = min(ymin * 2.0, max(self.height / 4.0, ymin)) if ymin <= self.height / 4.0 else 0.0
+            xmin = (
+                min(xmin * 2.0, max(self.width / 4.0, xmin))
+                if xmin <= self.width / 4.0
+                else 0.0
+            )
+            ymin = (
+                min(ymin * 2.0, max(self.height / 4.0, ymin))
+                if ymin <= self.height / 4.0
+                else 0.0
+            )
             xmax = self.width - xmin
             ymax = self.height - ymin
-            cursor = self.cursor
             for e in elements.elems(emphasized=True):
                 try:
                     if e.lock:
-                        self.cursor = wx.CURSOR_SIZING
+                        self.scene.cursor("sizing")
                         self.tool = self.tool_translate
-                        if self.cursor != cursor:
-                            self.scene.gui.SetCursor(wx.Cursor(self.cursor))
                         return RESPONSE_CHAIN
                 except (ValueError, AttributeError):
                     pass
             if xin >= xmax and yin >= ymax:
-                self.cursor = wx.CURSOR_SIZENWSE
+                self.scene.cursor("size_se")
                 self.tool = self.tool_scalexy_se
             elif xin <= xmin and yin <= ymin:
-                self.cursor = wx.CURSOR_SIZENWSE
+                self.scene.cursor("size_nw")
                 self.tool = self.tool_scalexy_nw
             elif xin >= xmax and yin <= ymin:
-                self.cursor = wx.CURSOR_SIZENESW
+                self.scene.cursor("size_ne")
                 self.tool = self.tool_scalexy_ne
             elif xin <= xmin and yin >= ymax:
-                self.cursor = wx.CURSOR_SIZENESW
+                self.scene.cursor("size_sw")
                 self.tool = self.tool_scalexy_sw
             elif xin <= xmin:
-                self.cursor = wx.CURSOR_SIZEWE
+                self.scene.cursor("size_w")
                 self.tool = self.tool_scalex_w
             elif yin <= ymin:
-                self.cursor = wx.CURSOR_SIZENS
+                self.scene.cursor("size_n")
                 self.tool = self.tool_scaley_n
             elif xin >= xmax:
-                self.cursor = wx.CURSOR_SIZEWE
+                self.scene.cursor("size_e")
                 self.tool = self.tool_scalex_e
             elif yin >= ymax:
-                self.cursor = wx.CURSOR_SIZENS
+                self.scene.cursor("size_s")
                 self.tool = self.tool_scaley_s
             else:
-                self.cursor = wx.CURSOR_SIZING
+                self.scene.cursor("sizing")
                 self.tool = self.tool_translate
-            if self.cursor != cursor:
-                self.scene.gui.SetCursor(wx.Cursor(self.cursor))
             return RESPONSE_CHAIN
         dx = space_pos[4]
         dy = space_pos[5]
@@ -190,31 +213,31 @@ class SelectionWidget(Widget):
             elements.set_emphasized_by_position(space_pos)
             if not elements.has_emphasis():
                 return RESPONSE_CONSUME
-            self.root.create_menu(
-                self.scene.context.gui, elements.first_element(emphasized=True)
+            create_menu(
+                self.scene.context.gui, elements.top_element(emphasized=True), elements
             )
             return RESPONSE_CONSUME
-        if event_type == "doubleclick":
+        elif event_type == "doubleclick":
             elements.set_emphasized_by_position(space_pos)
-            self.root.activate_selected_node()
+            self.scene.context.signal("activate_selected_nodes", 0)
             return RESPONSE_CONSUME
-        if event_type == "leftdown":
+        elif event_type == "leftdown":
             self.save_width = self.width
             self.save_height = self.height
             self.uniform = True
             self.tool(space_pos, dx, dy, -1)
             return RESPONSE_CONSUME
-        if event_type == "middledown":
+        elif event_type == "middledown":
             self.save_width = self.width
             self.save_height = self.height
             self.uniform = False
             self.tool(space_pos, dx, dy, -1)
             return RESPONSE_CONSUME
-        if event_type in ("middleup", "leftup"):
+        elif event_type in ("middleup", "leftup", "lost"):
             self.tool(space_pos, dx, dy, 1)
             self.elements.ensure_positive_bounds()
             return RESPONSE_CONSUME
-        if event_type == "move":
+        elif event_type == "move":
             if not elements.has_emphasis():
                 return RESPONSE_CONSUME
             if self.save_width is None or self.save_height is None:
@@ -246,7 +269,7 @@ class SelectionWidget(Widget):
             for e in elements.flat(types=("group", "file")):
                 e._bounds_dirty = True
             elements.update_bounds([b[0], b[1], position[0], position[1]])
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scalexy_se(self, position, dx, dy, event=0):
         """
@@ -279,7 +302,7 @@ class SelectionWidget(Widget):
             elements.update_bounds(
                 [b[0], b[1], b[0] + self.save_width, b[1] + self.save_height]
             )
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scalexy_nw(self, position, dx, dy, event=0):
         """
@@ -312,7 +335,7 @@ class SelectionWidget(Widget):
             elements.update_bounds(
                 [b[2] - self.save_width, b[3] - self.save_height, b[2], b[3]]
             )
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scalexy_ne(self, position, dx, dy, event=0):
         """
@@ -345,7 +368,7 @@ class SelectionWidget(Widget):
             elements.update_bounds(
                 [b[0], b[3] - self.save_height, b[0] + self.save_width, b[3]]
             )
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scalexy_sw(self, position, dx, dy, event=0):
         """
@@ -378,7 +401,7 @@ class SelectionWidget(Widget):
             elements.update_bounds(
                 [b[2] - self.save_width, b[1], b[2], b[1] + self.save_height]
             )
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scalex_e(self, position, dx, dy, event=0):
         """
@@ -403,7 +426,7 @@ class SelectionWidget(Widget):
             for e in elements.flat(types=("group", "file")):
                 e._bounds_dirty = True
             elements.update_bounds([b[0], b[1], position[0], b[3]])
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scalex_w(self, position, dx, dy, event=0):
         """
@@ -428,7 +451,7 @@ class SelectionWidget(Widget):
             for e in elements.flat(types=("group", "file")):
                 e._bounds_dirty = True
             elements.update_bounds([position[0], b[1], b[2], b[3]])
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scaley_s(self, position, dx, dy, event=0):
         """
@@ -453,7 +476,7 @@ class SelectionWidget(Widget):
             for e in elements.flat(types=("group", "file")):
                 e._bounds_dirty = True
             elements.update_bounds([b[0], b[1], b[2], position[1]])
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_scaley_n(self, position, dx, dy, event=0):
         """
@@ -478,7 +501,7 @@ class SelectionWidget(Widget):
             for e in elements.flat(types=("group", "file")):
                 e._bounds_dirty = True
             elements.update_bounds([b[0], position[1], b[2], b[3]])
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
 
     def tool_translate(self, position, dx, dy, event=0):
         """
@@ -498,7 +521,7 @@ class SelectionWidget(Widget):
                 e._bounds_dirty = True
             self.translate(dx, dy)
             elements.update_bounds([b[0] + dx, b[1] + dy, b[2] + dx, b[3] + dy])
-        self.scene.context.signal("refresh_scene", 0)
+        self.scene.request_refresh()
 
     def process_draw(self, gc):
         """
@@ -512,12 +535,22 @@ class SelectionWidget(Widget):
         bounds = elements.selected_area()
         matrix = self.parent.matrix
         if bounds is not None:
-            linewidth = 2.0 / matrix.value_scale_x()
-            self.selection_pen.SetWidth(linewidth)
-            font_size = 14.0 / matrix.value_scale_x()
+            try:
+                linewidth = 2.0 / matrix.value_scale_x()
+                font_size = 14.0 / matrix.value_scale_x()
+            except ZeroDivisionError:
+                matrix.reset()
+                return
+            try:
+                self.selection_pen.SetWidth(linewidth)
+            except TypeError:
+                self.selection_pen.SetWidth(int(linewidth))
             if font_size < 1.0:
                 font_size = 1.0  # Mac does not allow values lower than 1.
-            font = wx.Font(font_size, wx.SWISS, wx.NORMAL, wx.BOLD)
+            try:
+                font = wx.Font(font_size, wx.SWISS, wx.NORMAL, wx.BOLD)
+            except TypeError:
+                font = wx.Font(int(font_size), wx.SWISS, wx.NORMAL, wx.BOLD)
             gc.SetFont(font, wx.Colour(0x7F, 0x7F, 0x7F))
             gc.SetPen(self.selection_pen)
             x0, y0, x1, y1 = bounds
@@ -549,6 +582,7 @@ class RectSelectWidget(Widget):
 
     Rectangle Selection Widget, draws the selection rectangle if left-clicked and dragged
     """
+
     def __init__(self, scene):
         Widget.__init__(self, scene, all=True)
         self.selection_pen = wx.Pen()
@@ -610,14 +644,18 @@ class RectSelectWidget(Widget):
                         obj.node.emphasized = True
                     else:
                         obj.node.emphasized = False
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
             self.start_location = None
             self.end_location = None
             return RESPONSE_CONSUME
         elif event_type == "move":
-            self.scene.context.signal("refresh_scene", 0)
+            self.scene.request_refresh()
             self.end_location = space_pos
             return RESPONSE_CONSUME
+        elif event_type == "lost":
+            self.start_location = None
+            self.end_location = None
+            return RESPONSE_CONSUME
         return RESPONSE_DROP
 
     def process_draw(self, gc):
@@ -630,8 +668,13 @@ class RectSelectWidget(Widget):
             y0 = self.start_location[1]
             x1 = self.end_location[0]
             y1 = self.end_location[1]
-            linewidth = 3.0 / matrix.value_scale_x()
-            self.selection_pen.SetWidth(linewidth)
+            linewidth = 2.0 / matrix.value_scale_x()
+            if linewidth < 1:
+                linewidth = 1
+            try:
+                self.selection_pen.SetWidth(linewidth)
+            except TypeError:
+                self.selection_pen.SetWidth(int(linewidth))
             gc.SetPen(self.selection_pen)
             gc.StrokeLine(x0, y0, x1, y0)
             gc.StrokeLine(x1, y0, x1, y1)
@@ -646,6 +689,7 @@ class ReticleWidget(Widget):
     Draw the tracking reticles. Each different origin for the driver;position and emulator;position
     gives a new tracking reticle.
     """
+
     def __init__(self, scene):
         Widget.__init__(self, scene, all=False)
         self.reticles = {}
@@ -710,6 +754,7 @@ class LaserPathWidget(Widget):
 
     These are blue lines that track the previous position of the laser-head.
     """
+
     def __init__(self, scene):
         Widget.__init__(self, scene, all=False)
         self.laserpath = [[0, 0] for _ in range(1000)], [[0, 0] for _ in range(1000)]
@@ -735,7 +780,7 @@ class LaserPathWidget(Widget):
         self.laserpath_index = index
 
     def clear_laserpath(self):
-        self.laserpath = [[0,0] for _ in range(1000)], [[0,0] for _ in range(1000)]
+        self.laserpath = [[0, 0] for _ in range(1000)], [[0, 0] for _ in range(1000)]
         self.laserpath_index = 0
 
     def process_draw(self, gc):
@@ -756,6 +801,7 @@ class GridWidget(Widget):
     """
     Interface Widget
     """
+
     def __init__(self, scene):
         Widget.__init__(self, scene, all=True)
         self.grid = None
@@ -780,7 +826,11 @@ class GridWidget(Widget):
             _ = self.scene.context._
             if self.background is not None:
                 item = menu.Append(wx.ID_ANY, _("Remove Background"), "")
-                self.scene.gui.Bind(wx.EVT_MENU, lambda e: self.scene.gui.signal("background", None), id=item.GetId())
+                self.scene.gui.Bind(
+                    wx.EVT_MENU,
+                    lambda e: self.scene.gui.signal("background", None),
+                    id=item.GetId(),
+                )
                 if menu.MenuItemCount != 0:
                     self.scene.gui.PopupMenu(menu)
                     menu.Destroy()
@@ -791,16 +841,15 @@ class GridWidget(Widget):
         """
         Based on the current matrix calculate the grid within the bed-space.
         """
-        if self.scene.context is not None:
-            context = self.scene.context
+        context = self.scene.context
+        if context is not None:
             bed_dim = context.root
             wmils = bed_dim.bed_width * MILS_IN_MM
             hmils = bed_dim.bed_height * MILS_IN_MM
         else:
             wmils = 310 * MILS_IN_MM
             hmils = 210 * MILS_IN_MM
-
-        kernel_root = self.scene.context.root
+        kernel_root = context.root
         convert = kernel_root.units_convert
         marks = kernel_root.units_marks
         step = convert * marks
@@ -848,17 +897,21 @@ class GridWidget(Widget):
             if self.grid is None:
                 self.calculate_grid()
             starts, ends = self.grid
+            matrix = self.scene.widget_root.scene_widget.matrix
             try:
-                line_width = (
-                    1 / self.scene.widget_root.scene_widget.matrix.value_scale_x()
-                )
+                scale_x = matrix.value_scale_x()
+                line_width = 1.0 / scale_x
                 if line_width < 1:
                     line_width = 1
-                self.grid_line_pen.SetWidth(line_width)
+                try:
+                    self.grid_line_pen.SetWidth(line_width)
+                except TypeError:
+                    self.grid_line_pen.SetWidth(int(line_width))
+
                 gc.SetPen(self.grid_line_pen)
                 gc.StrokeLineSegments(starts, ends)
-            except OverflowError:
-                self.scene.widget_root.scene_widget.matrix.reset()
+            except (OverflowError, ValueError, ZeroDivisionError):
+                matrix.reset()
 
     def signal(self, signal, *args, **kwargs):
         """
@@ -876,6 +929,7 @@ class GuideWidget(Widget):
 
     Guide lines drawn at along the scene edges.
     """
+
     def __init__(self, scene):
         Widget.__init__(self, scene, all=False)
         self.scene.context.setting(bool, "show_negative_guide", True)
@@ -927,7 +981,7 @@ class GuideWidget(Widget):
                     ends.append((x, h - length - edge_gap))
 
                     # gc.DrawText("%g %s" % (mark_point, p.units_name), x, 0, -tau / 4)
-                    gc.DrawText("%g" % mark_point, x, edge_gap, -tau / 4)
+                    gc.DrawText("%g" % mark_point, x, edge_gap, - math.tau / 4)
             x += points
 
         y = offset_y
